//
// Generated from archetype; please customize.
//

package org.codehaus.xma.mojo.packager
import org.apache.maven.plugin.Mojo;

import org.codehaus.groovy.maven.mojo.GroovyMojo
import org.codehaus.plexus.component.configurator.ComponentConfigurator;
import org.codehaus.plexus.component.configurator.expression.ExpressionEvaluator;
import org.codehaus.plexus.configuration.PlexusConfiguration;
import org.codehaus.plexus.configuration.xml.XmlPlexusConfiguration;
import org.codehaus.plexus.util.ReflectionUtils;
import org.codehaus.plexus.util.xml.Xpp3Dom;
import org.codehaus.plexus.util.xml.Xpp3DomBuilder;



/**
 * Packager Mojo used to wrap and provide the existing xma ant facility as maven plugin.
 *
 * @goal package
 * @requiresDependencyResolution
 */
class PackageMojo extends GroovyMojo {
    /**
     * @parameter expression="${project}"
     * @required
     * @readonly
    */
    org.apache.maven.project.MavenProject project
   /**
     * @parameter expression="${session}"
     * @required
     * @readonly
    */
    org.apache.maven.execution.MavenSession session
	/**
     * @parameter expression="${project.artifactId}"
     */
    private String appDescriptorDirectory
	/**
     * @parameter expression="xma-app.xml"
     */
    private String appDescriptorFileName
    /**
     * @parameter
     * @required
     */
    private List components
    /**
     * @parameter
     */
    private List clientLibraries
    /**
     * @parameter
     * @required
     */
    private List clientIncludes
    /**
     * @parameter
     * @required
     */
    private List serverIncludes
    /**
     * @parameter expression="${project.build.directory}/xma/components"
     */
    private String componentsDirectory
    /**
     * @parameter expression="${project.build.directory}/xma/dependencies"
     */
    private String dependenciesDirectory
    /**
     * @parameter expression="${project.build.directory}/xma/web"
     */
    private String xmaWebDirectory
    /**
     * @parameter expression="${project.build.directory}/xma/web/components"
     */
    private String webComponentsDirectory
    /**
     * @parameter expression="${project.build.directory}/xma/web/clientrt"
     */
    private String webClientRtDirectory

    void execute() {
		//removing all registered build listeners, including default (that writes to console)
    	ant.project.getBuildListeners().each {
        	ant.project.removeBuildListener(it)
    	}
		// some preconditions..
		if (project.packaging!=/war/) {
			fail ("not a web project")
		}
		boolean fastDevelopmentMode = session.goals.any {it == "jetty:run" } || System.env["boot.fdm"]
		if (fastDevelopmentMode) {
	  		log.info  "openXma Fast-Development-Mode enabled"
		}
		// check appDescriptor
		def appDescriptorFile = new File(appDescriptorDirectory,appDescriptorFileName)
		if (!appDescriptorFile.exists()) {
			fail ("required appDescriptorFile '${appDescriptorFile.name}' not found in directory '${appDescriptorDirectory}'")
		}
        createBuildDirectories()
        copyDependencyArtifacts()
        jarComponentsAndCreateChecksum(fastDevelopmentMode)
	    jarClientIncludes()
        copyComponentsAndApplicationDescriptor(fastDevelopmentMode)

    }

    String createBuildDirectories() {
        log.info "components ${components}"
        log.info  "${project}"
        ant.mkdir(dir: dependenciesDirectory)
        ant.mkdir(dir: componentsDirectory)
		ant.mkdir(dir: "${webComponentsDirectory}")
		ant.mkdir(dir: "${webClientRtDirectory}")
        ant.taskdef(name: "xmachecksum", classname: "at.spardat.xma.boot.antext.XMAChecksum")
        ant.taskdef(name: "appdes",		 classname: "at.spardat.xma.boot.antext.AppDescriptor")
    }

    void copyDependencyArtifacts() {
        log.info "Package xma runtime dependencies.."
        project.dependencyArtifacts.plus(project.artifacts).unique().
            findAll({artifact -> !["test"].contains(artifact.scope) && !artifact.file.name.endsWith(".war") && !artifact.file.directory}).each({artifact ->
			def fileNameSplitted = artifact.file.name.split('\\.') as List
			def fileNameExtension= fileNameSplitted.last()
			fileNameSplitted = artifact.file.name.split('-') as List
			if (fileNameSplitted.size() <= 1) {
				log.warn("Artifact ${artifact.file} does not contain '-' between artifactname and version!")
			}
			def fileNameWithoutVersion = fileNameSplitted.first()
			ant.copy(tofile: "${dependenciesDirectory}/${fileNameWithoutVersion}.${fileNameExtension}",
				file:"${artifact.file}")
        })
    }
    
    void jarComponentsAndCreateChecksum(fastDevelopmentMode) {
        if (!fastDevelopmentMode) {
            log.info "Copy component jars.."
			components.each({ componentNameAndPackage ->
				def componentSplitted = componentNameAndPackage.split('=')
				def componentName = componentSplitted[0]
				def componentPackage = componentSplitted[1]
				log.info "Package client/server component '${componentName}'.."
				ant.jar(
					destfile: "${componentsDirectory}/${componentName}client.jar",
					basedir:  "${project.build.outputDirectory}",
					includes: "${componentPackage}/**",
					excludes: "${componentPackage}/**/server/*.*"
				)
				ant.xmachecksum(file: "${componentsDirectory}/${componentName}client.jar")
				ant.jar(
					destfile: "${componentsDirectory}/${componentName}server.jar",
					basedir:  "${project.build.outputDirectory}",
					includes: "${componentPackage}/**",
					excludes: "${componentPackage}/**/client/*.*"
				)

			})
		}
        log.info "Copy client libraries.."
		clientLibraries.each({ jarNameAndDestinationDirectory ->
			def splitted = jarNameAndDestinationDirectory.split('=')
    		def libraryName = splitted[0]
    		def destinationFolder = splitted[1]
			def destFolderFile = new File(xmaWebDirectory, destinationFolder)
			if (!destFolderFile.exists()) {
				ant.mkdir(dir: xmaWebDirectory + File.separator + destinationFolder)
			}
			ant.copy(tofile: "${xmaWebDirectory}/${destinationFolder}/${libraryName}.jar", file:"${dependenciesDirectory}/${libraryName}.jar")
            if (!fastDevelopmentMode) {
			    ant.xmachecksum(file: "${xmaWebDirectory}/${destinationFolder}/${libraryName}.jar")
            }
		})
    }

    void jarClientIncludes() {
        log.info "Package common client includes.."
		clientIncludes.each({ clientIncludePath -> log.info clientIncludePath })
            ant.jar(destfile: "${componentsDirectory}/commonclient.jar", update: true) {
                fileset(dir: "${project.build.outputDirectory}", includes: "${clientIncludes.join(',')}")
        }
        ant.xmachecksum(file: "${componentsDirectory}/commonclient.jar")
		if (false) {
			log.info "Package common server includes..\n${serverIncludes.join('\n')}"
			ant.jar(destfile: "${componentsDirectory}/commonserver.jar", update: true) {
				fileset(dir: "${project.build.outputDirectory}", includes: "${serverIncludes.join(',')}")
			}
		}
    }

    void copyComponentsAndApplicationDescriptor(fastDevelopmentMode) {
        log.info "Copy components jars.."
        if (new File("${dependenciesDirectory}/xmabootrt.zip" ).exists()) {
            ant.unzip(src:"${dependenciesDirectory}/xmabootrt.zip" ,dest:"${dependenciesDirectory}")
        }
		ant.copy(todir: "${webComponentsDirectory}", preservelastmodified:"true") {
			fileset(dir: "${componentsDirectory}", includes:"*.jar*", excludes:"*server.jar*")
		}
		ant.copy(todir : "${webClientRtDirectory}",	file : "${dependenciesDirectory}/xmartclient.jar",preservelastmodified : "true"	)
		ant.xmachecksum(file: "${webClientRtDirectory}/xmartclient.jar")

		ant.copy(todir : "${webClientRtDirectory}",	file : "${dependenciesDirectory}/epclient.jar",preservelastmodified : "true"	)
		ant.xmachecksum(file: "${webClientRtDirectory}/epclient.jar")

		log.info "Prepare xma application descriptor.."
		ant.copy(todir : "${xmaWebDirectory}",
			file : "${appDescriptorDirectory}/${appDescriptorFileName}",
			preservelastmodified : "true"	)
		ant.appdes(dir: "${xmaWebDirectory}", file: "${appDescriptorFileName}", fdm: fastDevelopmentMode)
    }
}


